@@ -417,6 +417,8 @@ WallScreen::moveViewport (int    x,
 			  int    y,
 			  Window moveWin)
 {
+    CompOption::Vector o(0);
+
     if (!x && !y)
 	return false;
 
@@ -458,6 +460,8 @@ WallScreen::moveViewport (int    x,
 
     determineMovementAngle ();
 
+    screen->handleCompizEvent ("wall", "start_viewport_switch", o);
+
     if (!grabIndex)
 	grabIndex = screen->pushGrab (screen->invisibleCursor (), "wall");
 
@@ -501,12 +505,99 @@ WallScreen::handleEvent (XEvent *event)
 
 	    moveViewport (-dx, -dy, None);
 	}
+	if (event->xclient.message_type == Atoms::xdndEnter)
+	{
+	    toggleEdges (true);
+	    edgeDrag = true;
+	}
+	else if (event->xclient.message_type == Atoms::xdndLeave)
+	    edgeDrag = false;
+
+	break;
+
+	case FocusIn:
+	case FocusOut:
+	    if (event->xfocus.mode == NotifyGrab)
+		poller.start ();
+	    else if (event->xfocus.mode == NotifyUngrab)
+		poller.stop ();
+	break;
+
+	case ConfigureNotify:
+
+	     if (event->xconfigure.window == screen->root ())
+		updateScreenEdgeRegions ();
+
 	break;
     }
 
     screen->handleEvent (event);
 }
 
+/*
+ * Borrowed this from PrivateScreen::updateScreenEdges
+ *
+ */
+
+#define SCREEN_EDGE_NUM		8
+
+void
+WallScreen::updateScreenEdgeRegions ()
+{
+    edgeRegion = CompRegion (0, 0, screen->width (), screen->height ());
+    noEdgeRegion = CompRegion (0, 0, screen->width (), screen->height ());
+
+    struct screenEdgeGeometry {
+	int xw, x0;
+	int yh, y0;
+	int ww, w0;
+	int hh, h0;
+    } geometry[SCREEN_EDGE_NUM] = {
+	{ 0,  0,   0,  2,   0,  2,   1, -4 }, /* left */
+	{ 1, -2,   0,  2,   0,  2,   1, -4 }, /* right */
+	{ 0,  2,   0,  0,   1, -4,   0,  2 }, /* top */
+	{ 0,  2,   1, -2,   1, -4,   0,  2 }, /* bottom */
+	{ 0,  0,   0,  0,   0,  2,   0,  2 }, /* top-left */
+	{ 1, -2,   0,  0,   0,  2,   0,  2 }, /* top-right */
+	{ 0,  0,   1, -2,   0,  2,   0,  2 }, /* bottom-left */
+	{ 1, -2,   1, -2,   0,  2,   0,  2 }  /* bottom-right */
+    };
+
+    for (unsigned int i = 0; i < SCREEN_EDGE_NUM; i++)
+    {
+	CompRegion edge (geometry[i].xw * screen->width () +
+			 geometry[i].x0,
+			 geometry[i].yh * screen->height () +
+			 geometry[i].y0,
+			 geometry[i].ww * screen->width () +
+			 geometry[i].w0,
+			 geometry[i].hh * screen->height () +
+			 geometry[i].h0);
+
+	noEdgeRegion -= edgeRegion;
+    }
+
+    edgeRegion -= noEdgeRegion;
+}
+
+#undef SCREEN_EDGE_NUM
+
+void
+WallScreen::positionUpdate (const CompPoint &pos)
+{
+    if (edgeDrag)
+	return;
+
+    if (edgeRegion.contains (pos))
+	toggleEdges (false);
+    else if (noEdgeRegion.contains (pos))
+    {
+	if (!screen->grabbed ())
+	    poller.stop ();
+	toggleEdges (true);
+    }
+}
+
 void
 WallWindow::activate ()
 {
@@ -534,6 +625,26 @@ WallWindow::activate ()
     window->activate ();
 }
 
+void
+WallWindow::grabNotify (int          x,
+			int          y,
+			unsigned int width,
+			unsigned int height)
+{
+    WallScreen::get (screen)->toggleEdges (true);
+    WallScreen::get (screen)->edgeDrag = true;
+
+    window->grabNotify (x, y, width, height);
+}
+
+void
+WallWindow::ungrabNotify ()
+{
+    WallScreen::get (screen)->edgeDrag = false;
+
+    window->ungrabNotify ();
+}
+
 void
 WallScreen::checkAmount (int 	      dx,
 			 int 	      dy,
@@ -698,7 +809,11 @@ WallScreen::initiateFlip (Direction         direction,
 	    return false;
     }
     else if (!optionGetEdgeflipPointer ())
+    {
+	toggleEdges (false);
+	poller.start ();
 	return false;
+    }
 
     switch (direction) {
     case Left:
@@ -1138,6 +1253,7 @@ WallScreen::preparePaint (int msSinceLastPaint)
 
     if (moving && curPosX == gotoX && curPosY == gotoY)
     {
+	CompOption::Vector o (0);
 	moving = false;
 	timer  = 0;
 
@@ -1149,6 +1265,8 @@ WallScreen::preparePaint (int msSinceLastPaint)
 	    if (!screen->grabExist ("switcher"))
 		screen->focusDefaultWindow ();
 	}
+
+	screen->handleCompizEvent ("wall", "end_viewport_switch", o);
     }
 
     cScreen->preparePaint (msSinceLastPaint);
@@ -1400,6 +1518,26 @@ WallScreen::createCairoContexts (bool initial)
     }
 }
 
+void
+WallScreen::toggleEdges (bool enabled)
+{
+    WALL_SCREEN (screen);
+
+    if (!enabled)
+    {
+	screen->removeAction (&ws->optionGetFlipLeftEdge ());
+	screen->removeAction (&ws->optionGetFlipUpEdge ());
+	screen->removeAction (&ws->optionGetFlipRightEdge ());
+	screen->removeAction (&ws->optionGetFlipDownEdge ());
+    }
+    else
+    {
+	screen->addAction (&ws->optionGetFlipLeftEdge ());
+	screen->addAction (&ws->optionGetFlipUpEdge ());
+	screen->addAction (&ws->optionGetFlipRightEdge ());
+	screen->addAction (&ws->optionGetFlipDownEdge ());
+    }
+}
 
 void
 WallScreen::optionChanged (CompOption           *opt,
@@ -1571,6 +1709,12 @@ WallScreen::WallScreen (CompScreen *screen) :
     setNotify (ArrowBaseColor);
     setNotify (ArrowShadowColor);
     setNotify (NoSlideMatch);
+    setNotify (EdgeflipPointer);
+
+    updateScreenEdgeRegions ();
+
+    poller.setCallback (boost::bind (&WallScreen::positionUpdate, this,
+				     _1));
 }
 
 WallScreen::~WallScreen ()
@@ -1599,6 +1743,12 @@ WallPluginVTable::init ()
 {
     if (!CompPlugin::checkPluginABI ("core", CORE_ABIVERSION))
         return false;
+    if (!CompPlugin::checkPluginABI ("composite", COMPIZ_COMPOSITE_ABI))
+	return false;
+    if (!CompPlugin::checkPluginABI ("opengl", COMPIZ_OPENGL_ABI))
+	return false;
+    if (!CompPlugin::checkPluginABI ("mousepoll", COMPIZ_MOUSEPOLL_ABI))
+	return false;
 
     return true;
 }
